%%
clc
clear;

%% read image and construct linear operator
img = im2double(imread('cat_noisy.jpg'));
imshow(img);
[ny, nx, c] = size(img);
n = ny*nx*c;
f = img(:);

% TODO: construct the linear operator for 3 channels.
e = ones(ny, 1);
A = spdiags([-e e], 0:1, ny, ny);
A(end, end) = 0;
Dy = kron(speye(nx), A);
e = ones(nx, 1);
B = spdiags([-e e], 0:1, nx, nx)';
B(:, end) = 0;
Dx = kron(B', speye(ny));
D = cat(1, kron(speye(3), Dx), kron(speye(3), Dy));

%% optimization part
% initialize the parameters
u = f;
max_iters = 100;

% huber parameter
epsilon = 0.1;

% weight for regularizor
rho = 0.6;

% parameters for line search
c1 = 0.001;
c2 = 0.02;
    
alpha_max = 100;
alpha0 = 0;  
% arrays to record the energy.
energy = zeros(1,max_iters);
norm_gradient = zeros(1, max_iters);

grad = computeFGrad(u, D, epsilon, f);
f0 = computeF(u, D, epsilon, f);

for i = 1:max_iters
     
    alpha = line_search(u, -grad'*grad, -grad, f0, D, epsilon, f, c1, c2, alpha_max, alpha0);
    
    u = u - alpha*grad;
    
    grad = computeFGrad(u, D, epsilon, f);
    f0 = computeF(u, D, epsilon, f);
    energy(i) = f0;
    norm_gradient(i) = norm(grad);
    fprintf('iteration: %d, energy: %.4e, norm of gradient: %.4e\n', i, energy(i), norm_gradient(i));
end
%% visualization
figure(1)
imshow(reshape(u, ny, nx,c));
figure(2)
subplot(1, 2, 1);
plot(energy);
subplot(1,2,2);
plot(norm_gradient);